import pymongo
import argh
import os
import stat

def read_keys(username: str, container_name: str = "overleafmongo", port: int = 27017,
              output_path: str = "/downloads") -> bool:
    """
    Read SSH public and private keys for a user from MongoDB and save to filesystem.
    
    Args:
        username: Username for SSH key files
        container_name: MongoDB container name
        port: MongoDB port
        output_path: Base path to save user SSH keys
    """
    print(f"Looking up user: {username}")
    
    # Connect to MongoDB
    client = pymongo.MongoClient(container_name, port)
    db = client.sharelatex
    users = db.users
    
    # Check if user exists and retrieve keys
    user_data = users.find_one({"email": username})
    if user_data is None:
        print("User not found")
        client.close()
        exit(1)
    
    # Check if SSH keys exist in the user document
    if "sshPublicKey" not in user_data or "sshPrivateKey" not in user_data:
        print("SSH keys not found for this user")
        client.close()
        exit(1)
        
    public_key = user_data["sshPublicKey"]
    private_key = user_data["sshPrivateKey"]
    
    # Save keys to filesystem if path and username are provided
    if output_path and username:
        try:
            # Create .ssh directory if it doesn't exist
            ssh_dir = os.path.join(output_path, username, ".ssh")
            os.makedirs(ssh_dir, exist_ok=True)
            
            # Define key file paths
            public_key_path = os.path.join(ssh_dir, "overleafcep.pub")
            private_key_path = os.path.join(ssh_dir, "overleafcep")
            authorized_keys_path = os.path.join(ssh_dir, "authorized_keys")

            # Write authorized keys
            with open(authorized_keys_path, 'w') as file:
                file.write(public_key)
            os.chmod(authorized_keys_path, 0o644)  # rw-r--r--
           
            # Write public key
            with open(public_key_path, 'w') as file:
                file.write(public_key)
            os.chmod(public_key_path, 0o644)  # rw-r--r--
            
            # Write private key with restricted permissions
            with open(private_key_path, 'w') as file:
                file.write(private_key)
            os.chmod(private_key_path, 0o600)  # rw-------
            
            print(f"SSH keys saved to {ssh_dir}")
            print(f"Public key: {public_key_path}")
            print(f"Private key: {private_key_path}")
            print(f"Authorized key: {authorized_keys_path}")
            
        except Exception as e:
            print(f"Error saving SSH keys: {e}")
            client.close()
            exit(1)
    
    client.close()
    exit(0)

if __name__ == "__main__":
    argh.dispatch_command(read_keys)

