import argh  # type: ignore
import pymongo
import bson
import os
import subprocess


def get_user_id(
    username: str, container_name: str = "overleafmongo", port: int = 27017
) -> bson.objectid.ObjectId | None:

    # Establish connection to MongoDB
    client = pymongo.MongoClient(container_name, port)

    try:
        # Access the 'sharelatex' database
        db = client.sharelatex

        user_query_filter = {"email": username}

        # Use find_one() to get a single document that matches the query
        selected_user = db.users.find_one(user_query_filter)

        if selected_user:
            client.close()
            return selected_user["_id"]

    finally:
        # Close the connection regardless of the outcome (success or failure in finding the user)
        client.close()

    return None


def get_project_list(
    user_id: bson.objectid.ObjectId,
    container_name: str = "overleafmongo",
    port: int = 27017,
) -> list[dict] | None:
    # Establish connection to MongoDB
    client = pymongo.MongoClient(container_name, port)

    try:
        # Access the 'sharelatex' database
        db = client.sharelatex

        project_query_filter = {
            "$or": [
                {"owner_ref": user_id},  # User is the owner
                {"collaberator_refs": user_id},  # User is a collaborator
                {"readOnly_refs": user_id},  # User has read-only access
            ]
        }

        # Fetch all projects for a given userId
        projects_cursor = db.projects.find(
            project_query_filter,
            {
                "name": 1,
                "lastUpdated": 1,
                "publicAccessLevel": 1,
                "archived": 1,
                "trashed": 1,
                "_id": 1,
            },
        )

        # Filter and map projects based on conditions
        filtered_projects = [
            {
                "_id": str(project["_id"]),
                "name": project["name"],
            }
            for project in projects_cursor
            if not (project.get("archived") or project.get("trashed"))
        ]

        return filtered_projects

    finally:
        # Close the connection regardless of the outcome (success or failure in finding the user)
        client.close()

    return None


def main(
    username: str,
    container_name: str = "overleafmongo",
    port: int = 27017,
):

    if username is None:
        exit(1)

    if len(username) == 0:
        exit(1)

    user_id: bson.objectid.ObjectId | None = get_user_id(
        username=username, container_name=container_name, port=port
    )

    if user_id is None:
        exit(1)

    projectlist = get_project_list(
        user_id=user_id, container_name=container_name, port=port
    )

    os.makedirs(f"/downloads/{username}/projects.git", mode=0o700, exist_ok=True)

    with open(
        os.path.join("/downloads/", f"{username}", "projects.git", "projects.txt"),
        "w",
    ) as file:
        if projectlist is not None:
            for entry in projectlist:
                file.write(f'{entry["_id"]} ; "{entry["name"]}"\n')

    if not os.path.isdir("/downloads/{username}/projects.git/.git"):

        subprocess.run(
            [f"cd /downloads/{username}/projects.git && /usr/bin/git init -q "],
            shell=True,
        )

    subprocess.run(
        [f"cd /downloads/{username}/projects.git && /usr/bin/git add --all "],
        shell=True,
    )
    subprocess.run(
        [
            f"cd /downloads/{username}/projects.git && /usr/bin/git commit -q -m 'by HajTex' "
        ],
        shell=True,
    )

    subprocess.run(
        [f"chmod -R 0700 /downloads/{username}/projects.git "],
        shell=True,
    )

    exit(0)


if __name__ == "__main__":
    argh.dispatch_command(main)
