import pymongo
import argh
import os

def main(username: str, container_name: str = "overleafmongo", port: int = 27017, 
         path: str = "/downloads/", ) -> None:
    """
    Store SSH public and private keys for a user in MongoDB.
    
    Args:
        username: Username for SSH key files
        container_name: MongoDB container name
        port: MongoDB port
        path: Base path containing user SSH directories
    """
    print(f"User email: {username}")
    
    # Connect to MongoDB
    client = pymongo.MongoClient(container_name, port)
    db = client.sharelatex
    users = db.users
    
    # Check if user exists
    search_result = users.find_one({"email": username})
    if search_result is None:
        print("User not found")
        client.close()
        exit(1)
    
    # Get SSH keys from files
    if path and username:
        try:
            public_keys_name = os.path.join(path, username, ".ssh", "overleafcep.pub")
            private_keys_name = os.path.join(path, username, ".ssh", "overleafcep")
            
            # Verify directories and files exist
            assert os.path.isdir(os.path.join(path, username, ".ssh"))
            assert os.path.isfile(public_keys_name)
            assert os.path.isfile(private_keys_name)
            
            # Read key files
            with open(public_keys_name, 'r') as file:
                public_key = file.read()
            with open(private_keys_name, 'r') as file:
                private_key = file.read()
                
            # Update user with SSH keys
            users.update_one(
                {"email": username},
                {"$set": {"sshPublicKey": public_key, "sshPrivateKey": private_key}}
            )
            print("SSH keys stored successfully")
            
        except AssertionError:
            print("Error: SSH directory or key files not found")
            client.close()
            exit(1)
        except Exception as e:
            print(f"Error reading SSH keys: {e}")
            client.close()
            exit(1)
    else:
        print("Path and username are required to store SSH keys")
        client.close()
        exit(1)
    
    client.close()
    exit(0)

if __name__ == "__main__":
    argh.dispatch_command(main)

